"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var update = require("immutability-helper");
/**
 * Llama al metodo onModelChanged con el nuevo objeto del modelo con los datos cambiados,
 * considerando que el modelo original es immutable
 */
function handleModelPropertyChanged(newValue, key, props) {
    console.warn("use handleFormChange ya que esta tipado");
    if (props.onChange) {
        var updateSpec = {};
        updateSpec[key] = { $set: newValue };
        var newModel = update(props.value, updateSpec);
        props.onChange(newModel);
    }
}
exports.handleModelPropertyChanged = handleModelPropertyChanged;
/**Llama al método props.onChange incluyendo un cambio en una propiedad de value
 * @param change Objeto parcial que indica las propiedades que cambiaron de value, junto con sus nuevos valores
 * @param props Propiedades del componente
 */
function handleChange(change, props) {
    console.warn("use handleFormChange ya que esta tipado");
    var newModel = Object.assign({}, props.value || {}, change);
    if (props.onChange)
        props.onChange(newModel);
}
exports.handleChange = handleChange;
function createChangeFunction(propsThunk, key) {
    var changeFunc = (function (value) {
        return (_a = {}, _a[key] = value, _a);
        var _a;
    });
    return function (value) {
        return __awaiter(this, void 0, void 0, function () {
            var props, change, newModel;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        props = propsThunk();
                        if (!props.onChange) return [3 /*break*/, 2];
                        change = changeFunc(value);
                        newModel = Object.assign({}, props.value || {}, change);
                        return [4 /*yield*/, props.onChange(newModel)];
                    case 1:
                        _a.sent();
                        _a.label = 2;
                    case 2: return [2 /*return*/];
                }
            });
        });
    };
}
/**
 * Devuelve una curry-function que toma el props, el nombre de una propiedad de value, y el nuevo valor a asignar en esa propiedad.
 * La segunda funcion siempre devuelve la misma instancia para el mismo nombre de propiedad

 * En su clase cree la funcion de cambio:
 * const handleChange = handleFormChange(() => this.props);
 *
 * En los componentes editores:
 * <Text value={this.props.text} onChange={handleChange("Nombre")} />
 *
 * @param propsThunk Funcion que devuelve un objeto con el valor actual y la función de cambio
 * @param key Nombre de la propiedad
 * @param value Nuevo valor de la propiedad.
 *
 */
function handleFormChange(propsThunk) {
    var cache = {};
    return function (key) {
        return cache[key] || (cache[key] = createChangeFunction(propsThunk, key));
    };
}
exports.handleFormChange = handleFormChange;
/**Devuelve una función que maneja el onChange de un control de una forma, donde la propiedad cambiada es una propiedad de props.value.
 * Siempre devuelve la misma instancia de la función para un changeFunction equivalente.
 * Esta es la manera recomendada de manejar el onChange en una forma ya que al devolver siempre la misma función para el mismo campo, evitamos redibujar el campo
 *
 * @param changeFunction Función que dado un nuevo valor devuelve un objecto parcial con la propiedad que se va a cambiar asignada al nuevo valor de esa propiedad. Esta función debe de ser pura
 * @param propsThunk Función que devuelve el value y el onChange de la forma
 * @param cache Objeto donde se va a almacenar el cache de funciones
 */
function onChange(changeFunction, propsThunk, cache) {
    console.warn("use handleFormChange ya que esta tipado");
    var keys = Object.keys(changeFunction("test"));
    if (keys.length != 1) {
        throw "La función del handleChange debe de ser pura y devolver un objeto con solo una propiedad";
    }
    var key = keys[0];
    if (cache[key]) {
        return cache[key];
    }
    else {
        var newFunc = function (newValue) {
            var props = propsThunk();
            var change = changeFunction(newValue);
            var newModel = Object.assign({}, props.value || {}, change);
            if (props.onChange) {
                props.onChange(newModel);
            }
        };
        cache[key] = newFunc;
        return newFunc;
    }
}
exports.onChange = onChange;
