import { FiltroOrden } from "./gridState";
import { CellData, GridCell } from "./baseTypes";
import { FiltroFunc, GridColumnFilter, ItemRowData } from "./tiposFiltros";
import { Grid2Column } from "../simpleGrid/column";
import { ObjMap } from "keautils";
/**Representa una fila del grid, despues de haber obtenido los valores con el getData */
/**Obtiene el filtro efectivo de la columna, si el filtro no esta definido en la columna se usa el selector getAutoFiltro */
export declare function getFiltro<TItem>(data: TItem[] | PromiseLike<TItem[]>, getData: (item: TItem) => CellData, filter: GridColumnFilter<any> | undefined): Promise<GridColumnFilter<any>>;
/**
 * Para una lista de elementos, obtiene la misma lista de elementos pero obtieniendo cada uno de los datos de las columnas con sus respectivos getCell y los datos de la busqueda,
 * esta estructura de datos se usa para acelerar las búsquedas ya que estos valores se tendran que calcular sólo cuando cambien los items, en lugar de cada vez que se realize una búsqueda
 * @param data
 */
export declare function getItemCellData<T>(data: T[], columnas: ObjMap<GetDataColumna<T>>): ItemRowData<T>[];
/**Aplica los filtros de columnas y el orden a los datos
 * @param searchFunc Función que se utiliza en la búsqueda por texto
*/
export declare function filtrarOrdenar<T>(data: ItemRowData<T>[], filtroOrden: FiltroOrden, columnas: ObjMap<FiltroOrdenColumna<T>>): ItemRowData<T>[];
/**Convierte una columna del grid a una columna FiltroOrdenColumna */
export declare function gridColumnToFiltroOrdenColumn<TItem>(data: TItem[] | PromiseLike<TItem[]>, value: Grid2Column<TItem, any>): Promise<FiltroOrdenColumna<TItem> & GetDataColumna<TItem>>;
export declare function compare(a: any, b: any): 0 | 1 | -1;
/**Función que se usa para la búsqueda por texto */
export interface TextSearchFunc<T> {
    (pattern: string, item: T, columnas: GetDataColumna<T>[]): boolean;
}
export interface GetDataColumna<T> {
    /**Función que obtiene el contenido de la columna dado el objeto en esta fila. La función toma dos parametros, el primero es la fila
    * y el segundo los datos compartidos por todos los items
   */
    cell: (row: T) => GridCell<any, any>;
}
export interface FiltroOrdenColumna<T> {
    filtro: FiltroFunc<CellData, {}>;
}
