export declare type ValidationRuleResult = boolean | string | null | undefined;
/**Una función de validación. Devuelve false o una cadena no vacia si hay un error de validación, devuelve true si no hay error de validacón*/
export interface Rule<T> {
    (value: T, key: string): PromiseLike<ValidationRuleResult> | ValidationRuleResult;
}
export declare type Rules<T> = {
    [K in keyof T]?: Rule<T>;
};
/**Contiene los mensajes de errores de las propiedades que tienen error */
export declare type Result<T> = {
    [K in keyof T]?: Promise<string | undefined>;
};
/**Determina si un resultado de validación tiene errores */
export declare function hasError<T>(validation: Result<T>): Promise<boolean>;
/**Evalua un conjunto de reglas de validación dado un objeto */
export declare function getErrors<T>(rules: Rules<T>, value: T): Result<T>;
/**Devuelve la cadena "Valor requerido" si value es null o undefined, si no devuelve true*/
export declare function required(value: any, key: string): true | "Valor requerido";
/**
 * Devuelve un nuevo conjunto de reglas el cual sólo valida a errores cuando se cumple la condición, es decir, si la condición no se cumple, el nuevo conjunto de reglas no devuelve errores
 * @param rules Conjunto de reglas base
 * @param condition Condición tal que sólo permitirá devulver errores si es true
 */
export declare function when<T>(condition: ((value: T) => boolean) | boolean, rules: Rules<T>): Rules<T>;
/**
 * Devuelve un nuevo conjunto de reglas que es el AND entre dos conjuntos de reglas
 */
export declare function and<T>(...rules: Rules<T>[]): Rules<T>;
