/// <reference types="react" />
/**Convierte un componente controlado en uno no controlado, almacenando los valores no controlados en los cookies si es que el consumidor establece el prop cookieKey */
export declare function MakeUncontrolledCookies<TProps extends object>(valueKey: keyof TProps, onChangeKey: keyof TProps): (component: React.ComponentClass<TProps>) => React.ComponentClass<TProps & {
    cookieKey?: string;
}>;
/**
 * Convierte un componente controlado en uno no controlado, para el cual si se le pasa un value != undefined, se comporta como controlado
 * @param valueKey Nombre de la propiedad del valor
 * @param onChangeKey Nombre de la propiedad de la función de cambio en la forma (value) => void
 */
export declare function MakeUncontrolled<TProps extends object>(valueKey: keyof TProps, onChangeKey: keyof TProps, initialValue: any): (Component: React.ComponentClass<TProps>) => React.ComponentClass<TProps>;
/**Vuelve un componente controlado uno no controlado, donde en el estado inicial no se le pasara ningun valor al props de value */
export declare function mapStateUncontrolled<Props extends {
    value?: any;
    onChange?: (x: any) => void;
}>(component: React.ComponentClass<Props>): React.ComponentClass<Props>;
