import React = require("react");
import interfaces = require("../interfaces");
import Field = require("./formField");

/**Un checkbox de un input simple */
class SimpleCheckbox extends React.PureComponent<interfaces.EditorProps<boolean | null | undefined, boolean>, {}> {
    handleChange = (event: React.FormEvent<HTMLInputElement>) => {
        if (this.props.onChange)
            this.props.onChange(event.currentTarget.checked);
    }

    render() {
        return (
            <input
                type="checkbox"
                checked={this.props.value || false}
                disabled={this.props.isReadOnly}
                onChange={this.handleChange} />
        );
    }
}

/**Un checkbox. Si no se establece un label el checkbox sera solamente el input, si se
 * establece un label el checkbox sera el checkbox estilizado para bootstrap
 */
export class Checkbox extends React.PureComponent<interfaces.FormEditProps<boolean | null | undefined, boolean>, {}> {

    render() {
        return (
            this.props.label ?
                <div className="checkbox">
                    <label>
                        <SimpleCheckbox value={this.props.value} isReadOnly={this.props.isReadOnly} onChange={this.props.onChange} /> <span style={this.props.error ? { color: "red" } : {}} >{this.props.label}</span>
                    </label>
                </div>
                :
                <SimpleCheckbox value={this.props.value} isReadOnly={this.props.isReadOnly} onChange={this.props.onChange} />

        );
    }
}
