
import * as ReactDateTime from "react-datetime";
import interfaces = require("../interfaces");
import React = require("react");
import Field = require("./formField");
import format = require("../format");
import { Text } from "./text";
import { HourMinuteComponent } from "./hour"

interface HorasMinutos {
    horas: number;
    minutos: number;
}

export interface DateTimeState {
    stringValue: string;
}

export interface DateTimeProps extends interfaces.FormEditProps<Date | string | null, Date | null> {
    /**True para mostrar tambien el selector de la hora */
    time?: boolean;
}

/**Selector de fecha y hora */
export class DateTime extends React.PureComponent<DateTimeProps, {}> {
    handleChange = (value: any | string) => {
        if (!this.props.onChange) return;
        if (!value)
            this.props.onChange(null);

        if (typeof (value) == "string") {

        } else {
            this.props.onChange(value.toDate());
        }
    }

    private handleHourChange = (horas: number | null, minutos: number | null) => {
        if (!this.props.onChange) return;

        const date = this.dateValue;
        if (!date) {
            this.props.onChange(null);
            return;
        } else {
            const newDate = new Date(date.getFullYear(), date.getMonth(), date.getDate(), horas || 0, minutos || 0);
            this.props.onChange(newDate);
        }
    }

    private get dateValue() {
        const origValue = this.props.value;
        const value =
            origValue == null ? null :
                typeof (origValue) == "string" ? new Date(origValue) : origValue;
        return value;
    }

    render() {
        const value = this.dateValue;
        const hours = value && value.getHours();
        const minutes = value && value.getMinutes();

        return (
            <Field.FormField label={this.props.label} error={this.props.error}>
                {

                    <div style={{ display: "flex" }} >
                        <div style={{ flex: "1 0 0" }}>
                            {
                                this.props.isReadOnly ?
                                    <Text value={(this.props.value ? format.FormatDate(this.props.value) : "Sin fecha")} isReadOnly={true} /> :
                                    <ReactDateTime
                                        locale="es"
                                        value={value || undefined}
                                        onChange={this.handleChange}
                                        dateFormat={"DD/MM/YYYY"}
                                        closeOnSelect={true}
                                        timeFormat={false} />
                            }
                        </div>
                        {
                            this.props.time &&
                            <div style={{ flex: "1 0 0" }}>
                                <HourMinuteComponent
                                    min={0}
                                    max={24}
                                    hours={hours}
                                    minutes={minutes}
                                    onChange={x => this.handleHourChange(x.hours, x.minutes)}
                                    readonly={this.props.isReadOnly}
                                />
                            </div>
                        }
                    </div>
                }
            </Field.FormField>
        );
    }
}