import React = require("react");
const fastInputDelay = 300;
export interface FastInputProps {
    value?: string;
    onChange?: (value: string) => void;
    className?: string;
    disabled?: boolean;
    style?: React.CSSProperties;
    type?: string;
    placeholder?: string;
    multiline?: boolean;
}

/**
 * Un input controlado se vuelve muy lento en android debido a que se estan actualizando constantemente sus
 * propiedades.
 * 
 * Este componente envuelve a un input no controlado y solo actualiza su valor cuando es necesario, logrando
 * que se pueda escribir de manera fluida en este en móviles
 */
export class FastInput extends React.Component<FastInputProps, {}> {
    handleRef = (input: HTMLInputElement) => {
        if (input) {
            input.addEventListener("input", this.handleOnChange as any);
            input.addEventListener("blur", this.handleBlur);
        } else if (this.input) {
            this.input.removeEventListener("input", this.handleOnChange as any);
            this.input.removeEventListener("blur", this.handleBlur);
        }
        this.input = input;
        this.setInputValue(this.props.value || "");
    };

    input: HTMLInputElement | null;

    componentWillReceiveProps(newProps: { value: string }) {
        const isFocused = this.input == document.activeElement;
        if (isFocused) {
            this.unbouncedRefresh(newProps.value);
        } else if (this.input) {
            this.input.value = newProps.value;
        }
    }
    propsCount: number = 0;

    unbouncedRefresh = (value: string) => {
        this.propsCount++;
        const currentCount = this.propsCount;
        const currentChangeCount = this.changeCount;
        setTimeout(() => {
            if (currentCount == this.propsCount && currentChangeCount == this.changeCount) {
                this.setInputValue(value);
            }
        }, fastInputDelay);
    }

    changeCount: number = 0;
    lastValue : string;

    private raiseOnChange = (value: string) => {
        if(this.props.onChange && this.lastValue !== value) {
            this.lastValue = value;
            this.props.onChange(value);
        }
    }

    handleOnChange = (e: React.ChangeEvent<HTMLInputElement>) => {
        this.changeCount++;
        const currentCount = this.changeCount;

        setTimeout(() => {
            if (currentCount == this.changeCount) {
               this.raiseOnChange(e.target.value);
            }
        }, fastInputDelay);

    }

    handleBlur = (e: Event) => {
        this.propsCount++;
        this.changeCount++;
        if (this.input) {
            this.raiseOnChange(this.input.value);
        }
    }

    /**
     * Establece el valor del input igual al valor de props.value
     * @param value Nuevo valor
     */
    setInputValue(value: string) {
        if (this.input && this.input.value != value) {
            this.input.value = value;
        }
    }

    render() {
        const props =
            this.props.onChange ?
                Object.assign({}, this.props, { value: undefined, onChange: undefined }) :
                Object.assign({}, this.props)
            ;

        const elementType = this.props.multiline ? "textarea" : "input";
        //Quitamos los props que no le pertenecen al textarea ni al input
        const { multiline, ...newProps } = props;
        
        const ref = (this.props.onChange && this.handleRef);
        const element = React.createElement(elementType, { ref, ...newProps } as any);
        return element;
    }
}