import React = require("react");
import * as rx from "rxjs";
import { componentToRx } from "react-rxk";
import { toObservable } from "keautils";
export interface FormFieldProps {
    /**Contenido del label */
    label?: string | JSX.Element;
    /**True para mostrar el campo con un error */
    error?: boolean | string | rx.Observable<boolean | string | undefined> | PromiseLike<boolean | string | undefined>;
    children: JSX.Element | null | false;
}

interface FieldErrorProps {
    error?: string | boolean;
}

class FieldError extends React.PureComponent<FieldErrorProps> {
    render() {
        const props = this.props;
        const error = props.error;
        const text = typeof error == "string" ? error : "";
        return this.props.error ? <span className="label label-danger pull-right" ><i className="fa fa-time" />{text} </span> : null;
    }
}
const FieldErrorRx = componentToRx(FieldError);

interface FieldLabelProps {
    /**Contenido del label */
    label?: string | JSX.Element;
    error?: rx.Observable<boolean | string | undefined> | boolean | string;
}
class FieldLabel extends React.PureComponent<FieldLabelProps> {
    render() {
        return (
            <span>
                {this.props.label}
                <FieldErrorRx error={this.props.error} />
            </span>
        )
    }
}

/**Un campo de una ventana con un label y un hijo de edición */
export class FormField extends React.PureComponent<FormFieldProps, {}> {
    render() {
        const label = this.props.label && ((typeof this.props.label == "string") ? <label>{this.props.label}</label> : this.props.label)
        return (
            label ?
                (<div className={"form-group "}>
                    <FieldLabel {... this.props} error={toObservable(this.props.error)} />
                    {this.props.children}
                </div>) : (this.props.children || null)
        );
    }
}