import React = require("react");
import Field = require("./formField");
import { Text, Number } from "./text";
import { formatNumber } from "keautils";
export interface HourMinuteProps {
    hours: number | null;
    minutes: number | null;
    onChange: (value: { hours: number | null, minutes: number | null }) => void;
    min?: number;
    max?: number;
    readonly?: boolean;
}
export class HourMinuteComponent extends React.PureComponent<HourMinuteProps> {
    render() {
        const props = this.props;


        // const maxMinutes = props.hours && props.max ? Math.min(59, (props.max - props.hours) * 60) : 59;
        return (
            <div style={{ display: "flex" }} >
                <div style={{ flex: "1 0 0" }}>
                    <Text
                        isReadOnly={props.readonly}
                        value={formatNumber(props.hours, 2)}
                        onChange={(x) => this.props.onChange({ 
                            hours: x ? Math.min(Math.max(parseInt(x), 0), 23) : null, 
                            minutes: this.props.minutes 
                        })}
                        disableFastInput
                    />
                </div>

                <div style={{ flex: "1 0 0" }}>
                    <Text
                        isReadOnly={props.readonly}
                        value={formatNumber(props.minutes, 2)}
                        onChange={(x) => this.props.onChange({ 
                            hours: this.props.hours, 
                            minutes: x ? Math.min(Math.max(parseInt(x), 0), 59) : null 
                        })}
                        disableFastInput
                    />
                </div>
            </div>
        );
    }
}

export interface Props {
    /**Cantidad de horas seleccionadas */
    value: number | null;
    onChange?: (value: number | null) => void;
    label?: string;
    error?: boolean;
    /**Valor minimo de las horas. Default es 0 */
    min?: number;
    /** */
    /**Valor máximo de las horas. Default es 24 */
    max?: number;
    readonly?: boolean;
}

export class Hours extends React.PureComponent<Props>{
    private handleOnChange = (value: { hours: number | null, minutes: number | null }) => {
        if (this.props.onChange) {
            var horas = value.hours || 0;
            var minutos = value.minutes || 0;

            var valor = (horas + (minutos / 60.0));
            this.props.onChange(valor);
        }

    };

    render() {
        const value = this.props.value;
        const hours = ((value != null) && (value < 0 ? Math.ceil(value) : Math.floor(value))) || 0;
        const minutes = ((value != null) && (hours != null) && Math.round(value * 60 - hours * 60)) || 0;

        return (
            <Field.FormField label={this.props.label} error={this.props.error}>
                <HourMinuteComponent
                    hours={hours}
                    minutes={minutes}
                    onChange={this.handleOnChange}
                    readonly={this.props.readonly}
                    min={this.props.min || 0}
                    max={this.props.max || 24}
                />
            </Field.FormField>)
    }
}
