
import { Text } from "./text";
import React = require("react");
import interfaces = require("../interfaces");
import Field = require("./formField");

export interface PhoneProps extends interfaces.FormEditProps<string | undefined | null, string> {
    /**True para ocultar el segundo cuadro de texto para la extensión telefónica, esta se grabará en el value con un separador de "ext."" */
    hideExtension?: boolean;
}

function parseValue(val: string): { phone: string, ext: string } {
    const phoneRegex = /((?:\d|-|\s|\t|,)+)\s*(?:ext\.\s+(.*))?/;
    if (phoneRegex.test(val)) {
        const result = phoneRegex.exec(val)!;
        return { phone: result[1], ext: result[2] };
    } else {
        return { phone: val, ext: "" };
    }
}

function valueToString(value: { phone: string, ext: string }) {
    return value.ext ? value.phone + " ext. " + value.ext : value.phone;
}
export class Phone extends React.PureComponent<PhoneProps> {
    private handlePhoneChange = (value: string) => {
        if (this.props.onChange) {
            this.props.onChange(valueToString({ ... this.parsedValue, phone: value }));
        }
    }

    private handleExtChange = (value: string) => {
        if (this.props.onChange) {
            this.props.onChange(valueToString({ ... this.parsedValue, ext: value }));
        }
    }

    get parsedValue(): { phone: string, ext: string } {
        return parseValue(this.props.value || "");
    }


    render() {
        const parse = this.parsedValue;
        return (
            <Field.FormField label={this.props.label} error={this.props.error}>
                <div style={{ display: "flex" }} >
                    <div style={{ flex: "3 0 0" }}>
                        <Text value={parse.phone} onChange={this.handlePhoneChange} placeholder="teléfono" />
                    </div>
                    {
                        !this.props.hideExtension &&
                        <div style={{ flex: "1 0 0" }}>
                            <Text value={parse.ext} onChange={this.handleExtChange} placeholder={"ext."} />
                        </div>
                    }
                </div>
            </Field.FormField>
        );
    }
}