import * as React from "react";

export interface Props {
    value?: string;
    onChange?: (value: string) => void;
    className?: string;
    disabled?: boolean;
    style?: React.CSSProperties;
    type?: string;
    placeholder?: string;
    multiline?: boolean;
    onFocus?: () => void;
    onBlur?: () => void;
    onKeyPress?: (ev: React.KeyboardEvent<HTMLInputElement>) => void;
    selectAllOnFocus?: boolean;
    onEnter?: () => void;
}

export class SlowInput extends React.PureComponent<Props> {
    private handleKeyPress = async (ev: React.KeyboardEvent<HTMLInputElement>) => {
        if (this.props.onKeyPress) {
            this.props.onKeyPress(ev);
        }
    };

    handleOnChange = (e: React.ChangeEvent<HTMLInputElement>) => {
        if (this.props.onChange)
            this.props.onChange(e.target.value);

    }

    render() {
        const props =
            this.props.onChange ?
                Object.assign({}, this.props, { value: undefined, onChange: undefined }) :
                Object.assign({}, this.props)
            ;
        const elementType = this.props.multiline ? "textarea" : "input";
        //Quitamos los props que no le pertenecen al textarea ni al input
        const { multiline, selectAllOnFocus, onEnter, ...newProps } = props;

        const element = React.createElement(elementType, {
            ...newProps,
            onKeyPress: this.handleKeyPress,
            onChange: this.handleOnChange,
            value: this.props.value
        } as any);
        return element;
    }
}