import * as React from "react";
import * as val from "./validation"
import { default as moize } from "moize";
import { handleFormChange } from "./form"

/**Toma los props de un form y devuelve una función que convierte cualquier componente editor de un valor a uno que ya este ligado a un campo de ese form. Ejemplo
 * const getField =  formFieds(miValor, miOnChange, miErrors);
 * const FormText = getField(kea.Text);
 * 
 * //FormText esta ligado su value, onChange y errors al de la forma
 * <FormText field="mi_campo" />
 */
export function formfields<T>(getProps: () => { value?: T, onChange?: (x: T) => void }, errors?: () => val.Result<T>) {
    const onChange = handleFormChange(getProps);
    const func = function <TProps>(Component: ((props: TProps) => JSX.Element | null) | React.ComponentClass<TProps>): React.ComponentClass<TProps & { field: keyof T }> {
        type OutProps = TProps & { field: keyof T };
        interface FieldProps {
            label?: string | JSX.Element;
            value?: any;
            onChange?: (value: any) => void;
            error?: any;
            field: keyof T;
        }

        class FieldComp extends React.Component<OutProps> {
            render() {
                const errs = errors ? errors() : {};
                const { value } = getProps();
                const fieldProps = this.props as FieldProps;
                const mixedProps: FieldProps = {
                    label: fieldProps.field,
                    value: value && value[fieldProps.field],
                    onChange: onChange(fieldProps.field),
                    error: errs[fieldProps.field],
                    ...fieldProps,
                };
                const { field, ...renderProps } = mixedProps;


                return <Component {... (renderProps as any) } />
            }
        }
        const ret: React.ComponentClass<OutProps> = FieldComp;
        const innerName = (Component as any).displayName || Component.name || "Component";
        ret.displayName = `FormField(${innerName})`;
        return ret;
    };
    const ret = moize(func);
    return ret as typeof func;
}