import React = require("react");
import { CellData, GridDataFormat, ColumnaProps } from "./tipos";
import { formatCellData } from "./format";
import { Link } from "../botones";

export class GridCellComponent extends React.PureComponent<{ data: CellData, format?: GridDataFormat }, {}> {
    render() {
        const text= formatCellData(this.props.data, this.props.format);
        return (
            <td title={text}>
                {text}
            </td>);
    }
}

/**Controles de las cabeceras del grid */
export class GridCellControls extends React.PureComponent<ColumnaProps, {}> {
    render() {
        const props = this.props;
        return (
            <span>
                {
                    props.orden &&
                    <Link
                        onClick={props.onOrdenar}
                        style={"info"}
                        iconClass={"fa fa-sort-" + props.orden} />
                }
                {
                    props.puedeFiltrar &&
                    <Link style={props.estaFiltrado ? "info" : undefined} iconClass="fa fa-filter" onClick={props.onFiltrar} />
                }
            </span>
        );
    }
}

const thStyle = { whiteSpace: "nowrap", cursor: "pointer" };
export class GridHeaderComponent extends React.PureComponent<ColumnaProps & { titulo: string }, {}> {
    render() {
        return (
            <th style={thStyle} >
                <span>
                    <span onClick={this.props.puedeOrdenar ? this.props.onOrdenar : undefined}>
                        {this.props.titulo}
                    </span>
                    <GridCellControls {...this.props} />
                </span>
            </th>
        );
    }
}