import React = require("react");
import { Text } from "../../fields/text";
import { Button } from "../../botones";
import { Grid2State, ColumnMapping } from "../simpleGrid/gridState";
import { MappingComponent } from "./mappings";
import { ObjMap } from "keautils";
import { ExportComp } from "./export";
export interface GridBusquedaProps {
    value: Grid2State;
    onChange: (value: Grid2State) => void;
    /**Cuando el usuario desea generar un CSV */
    onExpotar: () => Promise<void>;
    /**Nombres de todas las columnas efectivas, en el orden original */
    columnas: ObjMap<string>;
}


export interface State {
    showMappingModal: boolean;
    showExportarModal: boolean;
}
/**Compoonente que muestra el texto de búsqueda y el botón para descargar CSV */

export class GridBusquedaComponent extends React.PureComponent<GridBusquedaProps, State> {
    constructor(props) {
        super(props);
        this.state = { showMappingModal: false, showExportarModal: false };
    }

    private handleBusquedaChange = (value: string) => {
        const oldValue = this.props.value;
        const newValue: Grid2State = { ...oldValue, filtroOrden: { ...oldValue.filtroOrden, busqueda: value } };
        this.props.onChange(newValue);
    }

    private handleOnExportChange = (value: ColumnMapping) => {
        const oldValue = this.props.value;
        const newValue: Grid2State = { ...oldValue, export: value };
        this.props.onChange(newValue);
    }

    render() {
        const { props, state } = this;
        return (
            <div style={{ display: "flex", flexDirection: "row" }}>
                <div style={{ flex: "1 1 auto" }}>
                    <Text value={this.props.value.filtroOrden.busqueda}
                        placeholder="Búsqueda" onChange={this.handleBusquedaChange} addon={<i className="fa fa-search" />} />
                </div>
                <div style={{ flex: "0 1 auto" }}>
                    <Button tooltip="Columnas" style="info" iconClass="fa fa-columns" onClick={() => this.setState({ showMappingModal: true })} />
                </div>
                {
                    <div style={{ flex: "0 1 auto" }}>
                        <Button tooltip="Descargar CSV" style="success" iconClass="fa fa-download" onClick={() => this.setState({ showExportarModal: true })} />
                    </div>
                }

                <MappingComponent
                    columnas={props.columnas}
                    onClose={() => this.setState({ showMappingModal: false })}
                    onStateChange={props.onChange}
                    state={props.value}
                    show={state.showMappingModal}
                />

                {
                    this.state.showExportarModal &&
                    <ExportComp
                        columnas={props.columnas}
                        onCerrar={() => this.setState({ showExportarModal: false })}
                        mapping={props.value.export}
                        onMappingChange={this.handleOnExportChange}
                        onExportar={this.props.onExpotar}
                    />
                }
            </div>
        );
    }
}