import React = require("react");
import { CellData, SimpleCellData, CellView, OnDrillDownEvent } from "../simpleGrid/baseTypes";
import { cellToText } from "../simpleGrid/text";
import { ObjectCell } from "./cells/objectcell";
import { ArrayCell, ArrayCellDeps } from "./cells/arraycell";

/**Vista por default para una celda */
export const defaultCellView: CellView<CellData, any> = (data: CellData, sharedData: any, onDrillDown: OnDrillDownEvent, string: string) => {
    if (isSimpleCellData(data)) {
        return string == undefined ? cellToText(data) : string;
    } else {
        return objectCell(data, sharedData, onDrillDown, string);
    }
}

/**Vista para un objeto con capacitdad de hacer un drill down */
export const objectCell: CellView<{}, any> = (data: {}, sharedData: any, onDrillDown: OnDrillDownEvent) => {
    return <ObjectCell value={data} onDrillDown={onDrillDown} />
}

/**Función que devuelve un view dadas las dependencias de un array cell */
export const arrayCell: (deps?: ArrayCellDeps) => CellView<any[], any> = deps => (data, sharedData, onDrillDown, string) => {
    return <ArrayCell {...(deps || {}) } value={data} string={string} />
}


function isSimpleCellData(x: CellData): x is SimpleCellData {
    if (typeof (x) == "object" && x != null && !(x instanceof Date)) {
        return false;
    }
    //Comprobacion del compilador:
    const y: SimpleCellData = x;
    return true;
}