import React = require("react");
import { Link } from "../../../botones";
import { Modal } from "../../../modal";
import { Table } from "../../../table";
import { cellToText } from "../../index";

export interface ArrayCellColumn {
    title: string;
    cell: (item: any) => string;
}
export interface ArrayCellDeps {
    itemToString?: (item: any) => string;
    columns?: ArrayCellColumn[];
    separator?: string;
}
export interface Props extends ArrayCellDeps {
    value: any[] | undefined;
    string: string;
}

export interface State {
    showListModal?: boolean;
}
export class ArrayCell extends React.PureComponent<Props, State> {
    constructor(props) {
        super(props);
        this.state = {};
    }

    render() {
        const { value, columns, itemToString, separator, string } = this.props;
        const toString = itemToString || cellToText;
        const text = string;
        return (
            (value && value.length > 0) ?
                (<span>
                    <Link style="success" iconClass="fa fa-list" onClick={() => this.setState({ showListModal: true })} />
                    {text}
                    {
                        this.state.showListModal &&
                        <ArrayCellModal {... this.props} onClose={() => this.setState({ showListModal: false })} />
                    }
                </span>) : <span />
        );
    }
}

/**Modal que muestra el contenido del arreglo */
class ArrayCellModal extends React.PureComponent<Props & { onClose: () => void }> {
    render() {
        const columns = this.props.columns || [
            {
                cell: x => cellToText(x),
                title: ""
            }];

        const header =
            <tr>
                {columns.map(x => x.title).map(x => <th>{x}</th>)}
            </tr>;

        const row = (item: any) =>
            <tr>
                {columns.map(x => x.cell(item)).map(x => <td>{x}</td>)}
            </tr>;
        const rows = (this.props.value || []).map(row);
        return (
            <Modal
                buttonsWithKey={[]}
                onClose={this.props.onClose}
                title="Detalle"
            >
                <Table header={header} items={rows} />
            </Modal>
        );
    }
}