import React = require("react");
import { Link } from "../../botones";

export interface ColumnaProps {
    onOrdenar: () => void;
    /**Se llama para poner o quitar el filtro */
    onFiltrar: () => void;
    /**Se llama al quitar la columma */
    onQuitar: (() => void) | undefined;
    orden: "asc" | "desc" | undefined;
    puedeOrdenar: boolean;
    puedeFiltrar: boolean;
    /**True si la columna esta filtrada */
    estaFiltrado: boolean;
    titulo: string;

}

class GridColumnButtons extends React.PureComponent<ColumnaProps & { showButtons: boolean }> {

    render() {
        const props = this.props;
        return (
            <span style={{ opacity: this.props.showButtons ? 1 : 0 }} >
                {
                    props.orden &&
                    <Link
                        onClick={props.onOrdenar}
                        style={"info"}
                        iconClass={"fa fa-sort-" + props.orden} />
                }
                {
                    props.puedeFiltrar &&
                    <Link style={props.estaFiltrado ? "info" : undefined} iconClass="fa fa-filter" onClick={props.onFiltrar} />
                }
                {
                    props.onQuitar &&
                    <Link style="danger" iconClass="fa fa-times" onClick={props.onQuitar} />
                }
            </span>
        );
    }
}

const thStyle = { whiteSpace: "nowrap", cursor: "pointer" };
export class GridColumnComponent extends React.PureComponent<ColumnaProps, { visible: boolean }> {
    constructor(props) {
        super(props);
        this.state = { visible: false };
    }

    render() {
        const props = this.props;
        return (
            <th style={thStyle} onMouseEnter={() => this.setState({ visible: true })} onMouseLeave={() => this.setState({ visible: false })}  >
                <span>
                    <span onClick={this.props.puedeOrdenar ? this.props.onOrdenar : undefined}>
                        {this.props.titulo}
                    </span>
                    <GridColumnButtons {...this.props} showButtons={this.state.visible} />
                </span>
            </th>
        );
    }
}