import React = require("react");
import { Grid2Column } from "../simpleGrid/column";
import { FiltroColumnaMap } from "../simpleGrid/gridState";
import { FiltroComponent, GridColumnFilter, GridColumnFilterComponentProps, ItemRowData } from "../simpleGrid/tiposFiltros";
import { CellData, Key } from "../simpleGrid/baseTypes";
import { ObjMap, mapObject, filterObject } from "keautils";
import { getAutoFiltro } from "../filtros/autoFiltro";
import { async } from "../../async";
import { getFiltro } from "../simpleGrid/filtrarOrdenar";
import { Link } from "../../botones";
import { cellToText } from "../index";

export type Items<TItem> = PromiseLike<TItem[]> | TItem[];
export interface FiltroListProps<TItem, TShared> {
    value: FiltroColumnaMap
    onChange: (value: FiltroColumnaMap) => void;
    columnas: FiltroComponentColumnaMap;
    items:  PromiseLike<ItemRowData<TItem>[]>;
}

export interface FiltroComponentColumna<TItem> {
    /**Titulo de la columna*/
    title: string;
    key: string | number;
    /**Filtro que se va a usar en esta columna*/
    filterComponent: PromiseLike<FiltroComponent<any>>;
}

export interface FiltroComponentColumnaMap {
    [k: string]: FiltroComponentColumna<any>;
}



/**Convierte un GridColumn a un FiltroComponentColumna */
export function gridColumnToFiltroColumn<TItem>(data: TItem[] | PromiseLike<TItem[]>, value: Grid2Column<TItem, any>): FiltroComponentColumna<TItem> {
    const getData = (item: TItem) => value.cell(item).data;
    return {
        filterComponent: getFiltro(data, getData, value.filter).then(x => x.component),
        title: value.title,
        key: value.key || value.title
    };
}

interface FiltroComponentProps<TItem> {
    /**Items */
    items: PromiseLike<ItemRowData<TItem>[]>,
    /**Columna actual */
    columna: FiltroComponentColumna<TItem>,
    /**Datos del filtro */
    value: {} | undefined;
    /**Se ejecuta cuando se han cambiado los datos del filtro */
    onChange: (value: any) => void;
    /**Se ejecuta cuando el usuario quiere quitar el filtro */
    onQuitar: () => void;
}
/**Dibuja un solo filtro */
function filtroComponent<TItem>(props: FiltroComponentProps<TItem>) {
    return async(props.columna.filterComponent, Comp => {
        return async(props.items, items => <Comp
            items={items}
            column={props.columna.key}
            titulo={props.columna.title}
            value={props.value}
            onChange={props.onChange}
            onQuitar={props.onQuitar}
        /> ) 
    });

}

/**Componente que muestra todos los filtros de un grid */
export class FiltroListComponent extends React.PureComponent<FiltroListProps<any, any>> {
    private handleOnChange = (value: {}, key: Key) => {
        const props = this.props;
        const nuevo = { ...props.value, [key]: { datos: value } };
        props.onChange(nuevo);
    };

    private handleQuitar = (key: Key) => {
        const props = this.props;
        props.onChange(filterObject(props.value, (x, k) => k != key));
    }
    render() {
        const props = this.props;
        const comp =
            <div>
                <div className="row">
                    {
                        Object.keys(props.value).map((key) =>
                            <div className="col-md-4" key={key}>
                                {
                                    filtroComponent({
                                        items: props.items,
                                        columna: props.columnas[key],
                                        onChange: (value: {}) => this.handleOnChange(value, key),
                                        onQuitar: () => this.handleQuitar(key),
                                        value: props.value[key].datos
                                    })
                                }
                            </div>
                        )
                    }
                </div>
            </div>;
        return comp;
    }
}
