import React = require("react");
import { OrdenGrid, FiltroOrden, FiltroColumna, Grid2State } from "../simpleGrid/gridState";
import { GridColumnComponent } from "./column";
import { Grid2Column } from "../simpleGrid/column";
import { createSelector } from "reselect";
import { DrillDownColumn, FiltroColumnaMap } from "../simpleGrid/gridState";
import { quitarColumna } from "../simpleGrid/drillDowns";
import { Key } from "../simpleGrid/baseTypes";
import { omit } from "keautils";
export interface Props {
    value: Grid2State;
    onChange: (value: Grid2State) => void;
    /**Todas las columnas en el orden original */
    columnas: ColumnaHeaderMap;
}

export function gridColumnToColumnHeader(value: Grid2Column<any, any>): ColumnaHeader {
    return {
        titulo: value.title,
        puedeFiltrar: true
    }
}

export interface ColumnaHeader {
    puedeFiltrar: boolean;
    titulo: string;
}

export interface ColumnaHeaderMap {
    [key: string]: ColumnaHeader;
}

export class GridHeaderComponent extends React.PureComponent<Props> {
    private onChangeSelector = (props: Props) => props.onChange;
    private valueSelector = (props: Props) => props.value;
    private filtroOrdenSelector = createSelector(this.valueSelector, value => value.filtroOrden);
    private onFiltroOrdenChangeSelector = createSelector(this.onChangeSelector, this.valueSelector, (onChange, value) => (filtroOrden: FiltroOrden) => onChange({ ...value, filtroOrden }));
    private onColumnaChangeSelector = createSelector(this.onFiltroOrdenChangeSelector, this.filtroOrdenSelector, (onChange, value) => (columna: FiltroColumnaMap) => onChange({ ...value, columna: columna }));
    private onOrdenChangeSelector = createSelector(this.onFiltroOrdenChangeSelector, this.filtroOrdenSelector, (onChange, value) => (orden: OrdenGrid | undefined) => onChange({ ...value, orden }));
    private ordenSelector = createSelector(this.filtroOrdenSelector, x => x.orden);
    private filtrosColumnaSelector = createSelector(this.filtroOrdenSelector, x => x.columna);

    private estaFiltradaSelector = createSelector(this.filtrosColumnaSelector, filtros => (i: Key) => filtros[i] != null);
    private getOrdenColumnaSelector = createSelector(this.ordenSelector, orden => (i: Key) => {
        return (orden && orden.columna == i) ? orden.orden : undefined
    });

    private onQuitarColumnMappingSelector = createSelector(this.onChangeSelector, this.valueSelector,
        (onChange, value) => (index: Key) => onChange({ ...value, mapping: (value.mapping || []).filter(x => x != index) }));


    private onQuitarColumna = (index: Key) => {
        const newValue = quitarColumna(index, this.valueSelector(this.props));
        this.props.onChange(newValue);
    }

    private columnasSelector = (props: Props) => props.columnas;
    private mappingSelector = (props: Props) => props.value.mapping;
    /**Columnas ordenadas con el orden establecido por el usuario */
    private columnasOrdenadasSelector = createSelector(this.columnasSelector, this.mappingSelector, (columnas, mapping) => mapping.map(i => ({ key: i, columna: columnas[i] })));

    private columnasComponentSelector = createSelector(this.columnasOrdenadasSelector, this.estaFiltradaSelector, this.getOrdenColumnaSelector,
        (columnas, estaFiltrada, getOrdenColumna) =>
            columnas.map(col =>
                <GridColumnComponent
                    key={col.key}
                    puedeOrdenar={true}
                    puedeFiltrar={col.columna.puedeFiltrar}
                    estaFiltrado={estaFiltrada(col.key)}
                    onFiltrar={() => this.onFiltrar(col.key)}
                    onOrdenar={() => this.onOrdenar(col.key)}
                    orden={getOrdenColumna(col.key)}
                    titulo={col.columna.titulo}
                    onQuitar={() => this.onQuitarColumna(col.key)}
                />
            )
    );


    private onFiltrar = (index: Key) => {
        const filtros = this.filtrosColumnaSelector(this.props);
        const existe = filtros[index] != null;

        let nuevo: FiltroColumnaMap;
        if (existe) {
            //Quitar el filtro:
            nuevo = omit(filtros, [index as string]);
        } else {
            //Poner el filtro, iniciamos con los datos del filtro como indefinidos
            nuevo = { ...filtros, [index]: { datos: undefined } };
        }

        this.onColumnaChangeSelector(this.props)(nuevo);
    }

    private onOrdenar = (index: Key) => {
        const orden = this.ordenSelector(this.props);
        let nuevaDireccion: "asc" | "desc" | undefined;
        if (orden == undefined || orden.columna != index) {
            nuevaDireccion = "asc";
        } else if (orden.orden == "asc") {
            nuevaDireccion = "desc";
        } else {
            nuevaDireccion = undefined;
        }

        const nuevoOrden: OrdenGrid | undefined = nuevaDireccion ? { columna: index, orden: nuevaDireccion } : undefined;
        this.onOrdenChangeSelector(this.props)(nuevoOrden);
    }

    render() {
        const estaFiltrada = this.estaFiltradaSelector(this.props);
        const ordenColumna = this.getOrdenColumnaSelector(this.props);
        return (
            <tr>
                {this.columnasComponentSelector(this.props)}
            </tr>
        );
    }
}