import React = require("react");
import { Key } from "../simpleGrid/baseTypes";
import { ColumnMapping } from "../simpleGrid/gridState";
import { createSelector } from "reselect";
import { ObjMap, mapObject, enumObject, pipe, contains, upDownItem } from "keautils";
import { Link, Button } from "../../botones";
import { Table } from "../../table";
export interface Props {
    /**Titulos de todas las columnas que se estan mostrando, en el orden original */
    columnas: ObjMap<string>;

    /**Mapeo de las columnas que se va a mostrar */
    value: ColumnMapping;
    onChange: (value: ColumnMapping | undefined) => void;
}

export class ColumnMappingComponent extends React.PureComponent<Props> {
    private value = (props: Props) => props.value;
    private count = createSelector(this.value, x => x.length);
    private columnas = (props: Props) => props.columnas;

    /**Indices y titulos de las columnas que NO estan mapeadas */
    private columnasDisponibles = createSelector(this.value, this.columnas,
        (mapping, columnas) =>
            pipe(
                columnas,
                cols => enumObject(cols),
                cols => cols.filter(x => !contains(mapping, x.key)),
            )
    );

    /**Indices y titulos de las columnas que SI estan mapeadas, en el orden de mapeo */
    private columnasAgregadas = createSelector(this.value, this.columnas,
        (mapping, columnas) => mapping.map(key => ({ key, value: columnas[key] }))
    );

    private agregarColumna = (col: Key) => {
        const newValue = [... this.props.value, col];
        this.props.onChange(newValue);
    }

    private moverColumna = (indiceMapping: number, direccion: "up" | "down") => {
        const newValue = upDownItem(this.props.value, indiceMapping, direccion);
        this.props.onChange(newValue);
    }

    private quitarColumna = (indice: Key) => {
        const newValue = this.props.value.filter(x => x != indice);
        this.props.onChange(newValue);
    }

    private handleTodas = () => {
        this.props.onChange(undefined);
    }

    private handleNinguna = () => {
        this.props.onChange([]);
    }

    render() {
        const columnasDisponibles = this.columnasDisponibles(this.props).map(col =>
            <tr key={col.key}>
                <td>
                    <Link iconClass="fa fa-plus fa-fw" onClick={() => this.agregarColumna(col.key)} style="success" />
                </td>
                <td>{col.value ? col.value : <i>{col.key}</i>}</td>
            </tr>);

        const columnasAgregadas = this.columnasAgregadas(this.props).map((col, indiceMapping) =>
            <tr key={col.key}>
                <td>
                    <Link iconClass="fa fa-times fa-fw" onClick={() => this.quitarColumna(col.key)} style="danger" />
                    <Link iconClass="fa fa-arrow-up fa-fw" onClick={() => this.moverColumna(indiceMapping, "up")} />
                    <Link iconClass="fa fa-arrow-down fa-fw" onClick={() => this.moverColumna(indiceMapping, "down")} />
                </td>
                <td>{col.value ? col.value : <i>{col.key}</i>}</td>
            </tr>
        );

        const ninguna =
            <Button iconClass="fa fa-square-o" style="danger" onClick={this.handleNinguna} >
                Ninguna
            </Button>;
        const todas =
            <Button iconClass="fa fa-check-square-o fa-fw" style="success" onClick={this.handleTodas} >
                Todas
            </Button>
        return (
            <div style={{ display: "flex", flexDirection: "column" }}>
                <div style={{ flex: "0 1 auto" }}>
                    <div className="btn-group pull-right">
                        {ninguna}
                        {todas}
                    </div>
                </div>

                <div style={{ display: "flex", flex: 1 }}>
                    <div style={{ flex: 1 }}>
                        <label>Disponibles</label>
                        <Table
                            header={<tr><td /><td /></tr>}
                            items={columnasDisponibles}
                        />
                    </div>
                    <div style={{ flex: 1 }} >
                        <label>Seleccionadas</label>
                        <Table
                            header={<tr> <td /> <td /> </tr>}
                            items={columnasAgregadas}
                        />
                    </div>
                </div>
            </div>

        )
    }
}