import React = require("react");
import { ColumnMapping } from "../simpleGrid/gridState";
import { createSelector } from "reselect";
import { Modal } from "../../modal";
import { Table } from "../../table";
import { Link } from "../../botones";
import { Grid2State } from "../simpleGrid/gridState";
import { quitarColumna, getOriginalColumnCount } from "../simpleGrid/drillDowns";
import { Key } from "../simpleGrid/baseTypes";
import { ObjMap, mapObject, enumObject } from "keautils";
import { ColumnMappingComponent } from "./mappingComp";
export interface Props {
    /**Titulos de todas las columnas que se estan mostrando, en el orden original */
    columnas: ObjMap<string>;
    state: Grid2State;
    onStateChange: (value: Grid2State) => void;
    show: boolean;
    onClose: () => void;
}

/**Intercambia 2 elementos de un arreglo, si los indices dados estan afuera del arreglo, devuelve el arreglo sin modificar */
function swapItems<T>(array: T[], a: number, b: number) {
    const inside = (x: number) => x >= 0 && x < array.length;
    if (!inside(a) || !inside(b))
        return array;

    return array.map((x, i, arr) =>
        i == a ? arr[b] :
            i == b ? arr[a] :
                arr[i]);
}

/**Mueve un elemento del arreglo, si el elemento no se puede mover en esta dirección debido a que esta en la esquina del arreglo devuelve el arreglo sin modificar */
function moveItem<T>(array: T[], index: number, dir: "up" | "down") {
    return swapItems(array, index, index + (dir == "up" ? -1 : +1));
}

export class MappingComponent extends React.PureComponent<Props> {
   private handleOnMappingChange = (mapping: ColumnMapping) => {
       const oldValue = this.props.state;
       const newwValue  : Grid2State= {... oldValue, mapping};
       this.props.onStateChange(newwValue);
   }
    render() {
        const props = this.props;
        return (
            this.props.show ?
                <Modal
                    buttonsWithKey={[]}
                    onClose={this.props.onClose}
                    title="Seleccionar columnas"
                >
                    <ColumnMappingComponent 
                        columnas={props.columnas}
                        onChange={this.handleOnMappingChange}
                        value={props.state.mapping}
                    />
                </Modal> : null
        )
    }
}