import React = require("react");
import { Paginacion } from "../simpleGrid/gridState";
import selector = require("../../selector");

export interface PageSelectorValues {
    /**Cantidad de elementos */
    itemsCount: number;
    /**Cantidad de páginas */
    pageCount: number;
    /**Página seleccionada truncada */
    selectedPage: number;
    /**Tamaño de página seleccionada */
    selectedSize: number;
    /**Tamaños de páginas disponibles para seleccionar */
    pageSizes: number[];
}

export function getPageValues(itemsCount: number, value: Paginacion): PageSelectorValues {
    const commonPageSizes = [10, 15, 30, 50, 100, 200, 500, 1000, 2000, 5000, 10000];
    const pageSizes = [...commonPageSizes.filter(x => x < itemsCount), itemsCount];
    const selectedSize = (() => {
        const filtrados = pageSizes.filter(x => x <= value.selectedPageSize)
        return filtrados[filtrados.length - 1] || itemsCount;
    })();
    const pageCount = Math.ceil(itemsCount / selectedSize)
    const selectedPage = Math.min(value.selectedPage, pageCount - 1);

    return {
        itemsCount,
        pageCount,
        selectedPage,
        selectedSize,
        pageSizes
    };
}

export interface PaginatorSelectorProps {
    itemsCount: number;
    value: Paginacion;
    onChange: (value: Paginacion) => void;
}

export class PaginatorSelector extends React.PureComponent<PaginatorSelectorProps> {
    render() {
        const values = getPageValues(this.props.itemsCount, this.props.value);
        return (
            <span>
                <PageSelector count={values.pageCount} selected={values.selectedPage} onValueChanged={page => this.props.onChange({ ... this.props.value, selectedPage: page })} />
                <div className="pull-right">
                    <SizeSelector pageSizes={values.pageSizes} value={values.selectedSize} onChange={size => this.props.onChange({ ... this.props.value, selectedPageSize: size })} />
                </div>
            </span>
        );
    }
}


/**Propiedades del paginador */
interface PageSelectorProps {
    /**Cantidad de paginas */
    count: number;
    /**Pagina seleccionada, iniciando en 0 */
    selected: number;
    /**Se ejecuta cuando el usuario de la click a una pagina, el numero de pagina inicia en 0 */
    onValueChanged: (page: number) => void;
}

/**Un selector de pagina */
class PageSelector extends React.PureComponent<PageSelectorProps, {}>{
    render() {
        const paginas = selector.Rango(this.props.count).map(x => ({ value: x, label: x + 1, tooltip: "página " + (x + 1) }));
        return (
            <selector.PaginatedButtonSelector
                items={paginas}
                value={this.props.selected}
                onChange={value => this.props.onValueChanged(value as number)}
            />
        )
    }
}


interface SizeSelectorProps {
    pageSizes: number[];
    value: number;
    onChange: (value: number) => void;
}
class SizeSelector extends React.PureComponent<SizeSelectorProps> {
    render() {
        return (
            <selector.ButtonSelector
                items={this.props.pageSizes.map(x => ({ value: x, label: x, tooltip: x + " elementos por página" }))}
                value={this.props.value}
                onChange={this.props.onChange} />
        );
    }
}