import React = require("react");
import { Grid2Column, } from "../simpleGrid/column";
import { Key, CellView } from "../simpleGrid/baseTypes";
import { defaultCellView } from "./cell"
import { cellToText } from "../index";
import { getEffectiveString } from "../simpleGrid/text";
import { async } from "../../async";
import { ItemRowData } from "../simpleGrid/tiposFiltros";
export interface Props<TItem, TShared> {
    value: ItemRowData<TItem>;
    sharedValue?: TShared;
    columns: Grid2Column<TItem, TShared>[];
    onDrillDown: (column: Key, path: (string | number)[]) => void;
}

/**Obtiene el nodo de react que se va a mostrar en un elemento TD */
function getView(view: CellView<any, any> | undefined, key: string | number, data: any, sharedValue: any, onDrillDown: (key: string | number, path: (string | number)[]) => void, string: string | undefined) {
    const effectiveView = view || defaultCellView;
    const effectiveString = getEffectiveString(data, string);
    const element = effectiveView(data, sharedValue, path => onDrillDown(key, path), effectiveString)
    if (typeof element == "string") {
        return <span>{element}</span>;
    }
    else {
        return element;
    }
}

/**Una fila del grid */
export class GridRow extends React.PureComponent<Props<any, any>> {
    render() {
        const props = this.props;
        return (
            <tr>
                {
                    props.columns.map(x => ({ data: props.value.cell[x.key || x.title], title: x.title, key: x.key! })).map(x =>
                        <td
                            key={x.key}
                            style={x.data.style && x.data.style(props.sharedValue)}>
                            {
                                async(x.data.data, data =>
                                    getView(x.data.view, x.key, data, props.sharedValue, props.onDrillDown, x.data.string))
                            }
                        </td>)
                }
            </tr>
        )
    }
}