import React = require("react");
import { Table } from "../../table";
import { Grid2Column } from "../simpleGrid/column";
import { Paginacion, Grid2State, DrillDownColumn } from "../simpleGrid/gridState";
import { getPageValues, PaginatorSelector } from "./pageSelector";
import { GridRow } from "./row";
import { Spinner } from "../../cargando";
import { async } from "../../async";
import { addDrilldown } from "../simpleGrid/drillDowns";
import { Key } from "../simpleGrid/baseTypes";
import { ItemRowData } from "../simpleGrid/tiposFiltros";
export interface DummyTableProps {
    header: JSX.Element;
    value: Grid2State;
}
interface PaginatedTableProps<TItem, TShared> {
    header: JSX.Element;
    /**Los elementos ya filtrados y ordenados pero sin paginar */
    items: ItemRowData<TItem>[];
    getKey?: (item: TItem) => string | number;
    sharedData?: TShared;
    /**Columnas ya ordenadas y filtradas */
    columns: Grid2Column<TItem, TShared>[];
    /**Valor de la paginación */
    value: Grid2State;
    onChange: (value: Grid2State) => void;
}

function pageItems<T>(items: T[], selectedPage: number, selectedSize: number) {
    return items.slice(selectedPage * selectedSize, (selectedPage + 1) * selectedSize);
}

class GridTable extends React.PureComponent<PaginatedTableProps<any, any>> {
    private onDrillDown = (column: Key, path: (string | number)[]) => {
        const newValue = addDrilldown(column, path, this.props.value);
        this.props.onChange(newValue);
    }

    render() {
        const props = this.props;
        const { selectedPage, selectedSize } = getPageValues(props.items.length, props.value.paginacion);
        const pageStart = selectedPage * selectedSize;

        const pagedItems = pageItems(props.items, selectedPage, selectedSize);
        const tableItems = pagedItems.map((x, i) =>
            <GridRow
                key={props.getKey ? props.getKey(x) : i}
                columns={props.columns}
                sharedValue={props.sharedData}
                value={x}
                onDrillDown={this.onDrillDown}
            />
        )
        return (
            <div>
                {
                    props.items.length > 0 &&
                    <PaginatorSelector itemsCount={props.items.length} value={props.value.paginacion} onChange={paginacion => props.onChange({ ... this.props.value, paginacion })} />
                }
                <Table header={props.header} items={tableItems} />
            </div>
        );
    }
}

/**Una tabla vacia pero que si muestra las columnas, ademas de eso muestra el icono de cargando */
class CargandoTable extends React.PureComponent<DummyTableProps> {
    render() {
        const emptyGridProps: PaginatedTableProps<any, any> = {
            columns: [],            
            getKey: () => 0,
            header: this.props.header,
            items: [],
            onChange: () => { },
            sharedData: undefined,
            value: this.props.value,
        };
        return (
            <div>
                <GridTable {...emptyGridProps} />
                <Spinner />
            </div>
        );
    }
}

export interface AsyncPaginatedTableProps<TItem, TShared> {
    header: JSX.Element;
    /**Los elementos ya filtrados y ordenados pero sin paginar */
    items: PromiseLike<ItemRowData<TItem>[]>;
    getKey?: (item: TItem) => string | number;
    sharedData?: TShared;
    columns: Grid2Column<TItem, TShared>[];
    /**Valor de la paginación */
    value: Grid2State;
    onChange: (value: Grid2State) => void;
}

export class AsyncGridTable extends React.PureComponent<AsyncPaginatedTableProps<any, any>> {
    render() {
        return async(this.props.items,
            items => <GridTable {... this.props } items={items} />,
            { cargando: <CargandoTable {... this.props} /> })
    }
}