import React = require("react");
import { filtrarOrdenar, FiltroOrdenColumna, gridColumnToFiltroOrdenColumn, getItemCellData, GetDataColumna } from "../simpleGrid/filtrarOrdenar";
import { FiltroOrden } from "../simpleGrid/gridState";
import { GridHeaderComponent, gridColumnToColumnHeader } from "./header";
import { gridColumnToFiltroColumn } from "./filtros";
import { AsyncGridTable } from "./table";
import { Grid2Column, ColumnMap } from "../simpleGrid/column";
import { Grid2State } from "../index";
import { createSelector } from "reselect";
import { mapObject, ObjMap, promiseAllObj } from "keautils";
import { ItemRowData } from "../simpleGrid/tiposFiltros";


export interface GenericProps<T> {
    items: PromiseLike<ItemRowData<T>[]>;
    effectiveColumns: ColumnMap;
    filtroOrdenColumnas: PromiseLike<ObjMap<FiltroOrdenColumna<T> & GetDataColumna<T>>>;
    getKey?: (item: T) => string | number;
    sharedProps?: any;
    gridState: Grid2State;
    onGridStateChange: (value: Grid2State) => void;
}
export type Props = GenericProps<any>;
export class GridTableFiltro extends React.PureComponent<Props> {
    private effectiveColumns = (props: Props) => props.effectiveColumns;

    private columnHeaderSelector = createSelector(this.effectiveColumns, cols => mapObject(cols, gridColumnToColumnHeader));
    private itemsSelector = (props: Props) => props.items;
    private filtroOrdenSelector = (props: Props) => props.gridState.filtroOrden;
  
    private itemsFiltroData = (props: Props) => props.items;
    private filtroOrdenColumnsSelector = ( props: Props)  => props.filtroOrdenColumnas;

    private itemsFiltradosSelector = createSelector(this.itemsFiltroData, this.filtroOrdenSelector, this.filtroOrdenColumnsSelector,
        async (items, orden, columns) => filtrarOrdenar(await items, orden, await columns));

    private mapping = (props: Props) => props.gridState.mapping;
    private mappedColumns = createSelector(this.effectiveColumns, this.mapping, (cols, mapping) => mapping.map(key => cols[key]));

    render() {
        const items = this.itemsFiltradosSelector(this.props);
        const props = this.props;

        //Cabecera del grid, que es capaz de editar el filtro orden 
        const header = <GridHeaderComponent
            columnas={this.columnHeaderSelector(this.props)}
            value={this.props.gridState}
            onChange={this.props.onGridStateChange}
        />;

        return (
            <AsyncGridTable
                columns={this.mappedColumns(this.props)}
                getKey={props.getKey}
                header={header}
                items={items}
                onChange={props.onGridStateChange}
                sharedData={props.sharedProps}
                value={props.gridState}
            />);
    }
}