export type Key = string | number;
/**Tipos de cell data sin incluir a los objetos */
export type SimpleCellData = string | number | boolean | Date | null | undefined;
/**Tipos de cell data incluyendo objetos */
export type CellData = SimpleCellData | {} | {}[] | SimpleCellData[];
/**Funcion onDrillDown */
export interface OnDrillDownEvent {
    (path: Key[]): void;
}
export interface CellView<TCellData, TShared> {
    (cellData: TCellData, itemData: TShared, onDrillDown: OnDrillDownEvent, string: string): JSX.Element | string | null;
}
/**Define a una celda del grid */
export interface GridCell<TCellData, TShared> {
    /**Datos de la celda que se usarán para fines de filtrado */
    data: TCellData;

    /**Estilo que se va a aplicar al elemento de la celda */
    style?: (itemData: TShared) => React.CSSProperties;

    /**Representación de cadena de la celda. Esto se usará para la exportación y para mostrar el contenido en la vista */
    string?: string;


    /**Función que obtiene los datos que se van a encerrar en la celda de la tabla o undefined para mostrar la vista por default.
   * La función toma los datos de la celda y los datos compartidos de los items.
   * 
   * Por default se utiliza la función defaultCellView la cual convierte la celda a texto o muestra un drill down en caso de que la celda sea un objecto.
   * Se incluyen las siguientes funciones: @see cellToText @see objectCell @see arrayCell
   * */
    view?: CellView<any, TShared>;
}
