import { Grid2Column } from "./column";
import { Link, BootstrapButtonStyle } from "../../botones";
import * as React from "react";

export interface BotonesListaDeps<T> {
    onEdit: (x: T) => void;
    onBorrar: (x: T) => void;
}

/**
 * Dadas las funciones de editar y borrar devuelve un conjunto de botones de grid con los botones de editar y borrar
 * @param deps 
 */
export function columnaBotonesDefault<T>(deps: BotonesListaDeps<T>): BotonGrid<T>[] {
    return [
        { iconClass: "fa fa-pencil", style: "info", tooltip: "Editar", onClick: deps.onEdit },
        { iconClass: "fa fa-trash-o", style: "danger", tooltip: "Eliminar", onClick: deps.onBorrar },
    ]
}

/**Una definición de un botón para un grid */
export interface BotonGrid<T> {
    onClick: (item: T) => void;
    tooltip?: string;
    iconClass?: string;
    disabled?: boolean;
    style?: BootstrapButtonStyle;
}

/**Crea una columna para el grid formada por un conjunto de botones */
export function columnaBotones<T>(botones: (BotonGrid<T> | false)[]): Grid2Column<T> {
    const filtrados = botones.filter(x => x !== false).map(x => x as BotonGrid<T>);
    const componentes = (x: T) =>
        filtrados.map((bot, i) =>
            <Link key={i}
                iconClass={bot.iconClass || "fa fa-cicle"}
                disabled={bot.disabled}
                onClick={() => bot.onClick(x)}
                tooltip={bot.tooltip}
                style={bot.style}
            />);

    return {
        title: "",
        key: "_botones",
        cell: x => ({
            data: undefined,
            view: () =>
                <span>
                    {componentes(x)}
                </span>
        })
    };

}