import * as React from "react";
import { JsonNodeComponent } from "./node";
import { TreeComponent, TreeState, TreePath } from "../treeview/index";
import { valueToJsonTreeView, JsonValue } from "./logic";
import { createSelector } from "reselect";
import { MakeUncontrolled } from "../HOC";
export interface Props {
    /**Función que se desea ejecutar cuando el usuario de clic en una propiedad */
    onPropClick?: (path: (string | number)[]) => void;
    /**Valor que se quiere mostrar */
    value: any;
    /**Estado del tree */
    state?: TreeState;
    /**Determina cuando hac cambiado el estado del tree */
    onStateChanged?: (value: TreeState) => void;
}


class JsonViewComponentControlled extends React.PureComponent<Props> {
    private valueSelector = (props: Props) => props.value;
    private onPropClickSelector = (props: Props) => props.onPropClick;
    private treeSelector = createSelector(this.valueSelector, value => [valueToJsonTreeView(value)]);

    private rendererSelector = createSelector(this.onPropClickSelector,
        onPropClick => (value: JsonValue, path: TreePath) => {
            const key = path.length == 0 ? "" : path[path.length - 1];
            return <JsonNodeComponent key={key} propname={key} value={value} onClick={onPropClick && (() => onPropClick(path.slice(1)))} />;
        });


    render() {
        return <TreeComponent
            onStateChange={this.props.onStateChanged!}
            state={this.props.state!}
            rowRenderer={this.rendererSelector(this.props)}
            value={this.treeSelector(this.props)}
        />
    }
}

export const initialStateTree: TreeState = {
    collapsed: []
};
export const JsonViewComponent = MakeUncontrolled<Props>("state", "onStateChanged", initialStateTree)(JsonViewComponentControlled);