import React = require('react');
import botones = require("./botones");

/**Propiedades de un modal */
export interface DialogProps {
    /**Titulo del cuadro de dialogo*/
    title: any;
    /**Ocurre cuando el dialogo es cerrado */
    onClose?: () => void;
    /**Botones del cuadro de dialogo. Ponga aqui los botones del mismo */
    buttonsWithKey?: JSX.Element | JSX.Element[];
    fullScreen?: boolean;
}

/**Componente que muestra un cuadro de dialogo. Establezca como hijos el cuerpo del dialogo */
export class Modal extends React.Component<DialogProps, {}> {
    private modalDiv: HTMLDivElement;
    constructor(props: DialogProps) {
        super(props);
    }

    private lastModalState: boolean = false;
    private onDialogClick = (ev: React.MouseEvent<HTMLDivElement>) => {
        ev.stopPropagation();
    }

    render() {
        const fullScreenStyleDialog = this.props.fullScreen ? { width: "100%", height: "100%", margin: 0, padding: 0 } : {};
        const fullScreenStyleContent = this.props.fullScreen ? { width: "100%", height: "100%", boxShadow: "0 0 0 0" } : {};
        const fullScreenStyleBody = this.props.fullScreen ? { padding: 0 } : {};
        return (
            <div style={{
                position: "fixed",
                background: "rgba(0,0,0,0.5)",
                top: 0,
                left: 0,
                right: 0,
                bottom: 0,
                zIndex: 10000
            }}
                onClick={this.props.onClose}
            >
                <div
                    className="modal"
                    role="dialog"
                    ref={e => this.modalDiv = e!}
                    style={{ display: "block", zIndex: 10001 }}
                >
                    <div className="modal-dialog" style={fullScreenStyleDialog} onClick={this.onDialogClick}   >
                        <div className="modal-content" style={Object.assign({
                            display: "flex",
                            flexFlow: "column nowrap",
                            minHeight: "calc(100vh - 200px)", 
                            maxHeight: "calc(100vh - 60px)",
                        }, fullScreenStyleContent)} >
                            <div className="modal-header">
                                <button type="button" className="close" onClick={this.props.onClose}>&times;</button>
                                <h4 className="modal-title">{this.props.title}</h4>
                            </div>
                            <div className="modal-body" style={Object.assign({
                                flex: 1,
                                display: "flex",
                                flexFlow: "column",
                                overflowY: "auto" as "auto",
                            }, fullScreenStyleBody)}>
                                {this.props.children}
                            </div>
                            <div className="modal-footer">
                                <div>
                                    {this.props.buttonsWithKey}
                                    <botones.CancelarButton onClick={this.props.onClose} />
                                </div>
                            </div>
                        </div>
                    </div>
                </div>
            </div>
        );
    }
}


export interface BorrarModalProps {
    /**Nombre de la entidad que se va a borrar*/
    entityName: string;
    /**Nombre del tipo de entidad que se va a borrar */
    entityTypeName: string;
    /**Accion que se va a ejecutar al borrar el modal */
    onDelete?: () => PromiseLike<void> | void;
    /**Accion que se ejecuta al cerrar el cuadro de dialogo */
    onClose?: () => void;
}

/**Modal para borrar un elemento */
export class BorrarModal extends React.Component<BorrarModalProps, {}>
{
    constructor(props) {
        super(props);
        this.handleOnDelete = this.handleOnDelete.bind(this);
    }

    handleOnDelete = async () => {
        if (this.props.onDelete)
            await this.props.onDelete();

        if (this.props.onClose)
            this.props.onClose();
    }

    render() {
        return (
            <Modal
                title={<span>Borrar {this.props.entityTypeName} <b>{this.props.entityName}</b></span>}
                buttonsWithKey={[<botones.BorrarButton key="borrar" onClick={this.handleOnDelete} />]}
                onClose={this.props.onClose}
            >
                ¿Borrar este {this.props.entityTypeName}?
            </Modal>
        );
    }
}


