import React = require('react');

import { NavBarItem, NavBarLink,  dark, light } from "./interfaces";
import { mapNavBarItem } from "./item";
export interface BaseNavBaseContainerProps {
    header?: NavBarItem[];
    /**Arreglo con el contenido de los items que se van a ver en el lado izquierdo de la barra */
    left?: NavBarItem[];

    /**Arreglo con el contenido de los items que se van a ver en el lado derecho de la barra */
    right?: NavBarItem[];
    height: number;
}
export interface NavBarContainerProps extends BaseNavBaseContainerProps {

    collapsed?: boolean;
}


class ExtendedNavBar extends React.PureComponent<BaseNavBaseContainerProps, {}>  {
    get style() {
        return {
            top: 0,
            height: (this.props.height + 1) + "px",
            left: 0,
            right: 0,
            position: "relative",
            background: dark,
            display: "flex",
        } as React.CSSProperties;

    }

    render() {
        return (
            /**El primer div se usa para ocupar espacio en la pagina pero el que se dibuja en realidad es el 2do div */
            <div style={this.style}>
                <div style={{ flex: "0 1 auto" }}>
                    {this.props.header && this.props.header.map((x, i) => mapNavBarItem(false, this.props.height, undefined, x, i))}
                    {this.props.left && this.props.left.map((x, i) => mapNavBarItem(false, this.props.height, undefined, x, i))}
                </div>

                <div style={{ flex: "1 0 auto" }}>
                </div>

                <div>
                    {this.props.right && this.props.right.map((x, i) => mapNavBarItem(false, this.props.height, undefined, x, i))}
                </div>
            </div>);
    }
}


class Burger extends React.Component<{ dark: boolean }, {}> {
    render() {
        const spanStyle: React.CSSProperties = {
            height: "3px",
            background: dark ? "black" : "white",
            width: "20px",
            margin: "3px 2px 3px 2px"
        };

        return (
            <div style={{ display: "inline-block" }}>
                <div style={spanStyle} />
                <div style={spanStyle} />
                <div style={spanStyle} />
            </div>
        );
    }
}


class CollapsedNavBar extends React.Component<BaseNavBaseContainerProps, { expanded: boolean }> {
    constructor(props) {
        super(props);

        this.state = { expanded: false };
    }

    private navBarStyle: React.CSSProperties = {
        top: 0,
        height: (this.props.height + 1) + "px",
        left: 0,
        right: 0,
        position: "absolute",
        background: dark,
        display: "flex",
    };

    private mapItem = (x, i) => mapNavBarItem(true, this.props.height, this.handleCollapse, x, i);

    private handleExpand = () => {
        this.setState(prev => ({ expanded: !prev.expanded }));
    }
    private handleCollapse = () => {
        this.setState({ expanded: false });
    }

    render() {
        return (
            <div style={{ top: 0, left: 0, right: 0 }} >
                <div style={{ ... this.navBarStyle, flexDirection: "column", height: undefined, position: "relative" }}>
                    <div>
                        <div style={{ flex: "0 1 auto" }}>
                            <div>
                                <div style={{ display: "flex" }}>
                                    <div style={{ flex: "1 0 auto" }} >
                                        {this.props.header && this.props.header.map(this.mapItem)}
                                    </div>
                                    <div onClick={this.handleExpand} style={{ flex: "0 0 auto", display: "flex", justifyContent: "center", flexDirection: "column", background: light }} >
                                        <Burger dark={true} />
                                    </div>
                                </div>
                            </div>
                        </div>

                        <div style={{ flex: "1 0 auto" }}>
                        </div>

                        <div>
                        </div>
                    </div>
                    {
                        this.state.expanded &&
                        <div>
                            {this.props.left && this.props.left.map(this.mapItem)}
                            {this.props.right && this.props.right.map(this.mapItem)}
                        </div>
                    }
                </div>
            </div>
        )
    }
}

export class NavBarContainer extends React.Component<NavBarContainerProps, {}> {
    render() {
        return (
            this.props.collapsed ?
                <CollapsedNavBar {... this.props} />
                : <ExtendedNavBar {... this.props} />
        );
    }
}