import React = require("react");
import { FormEditProps } from "./interfaces";
import { SignatureCanvas, Stroke } from "./signatureCanvas";
import { FormField } from "./fields/formField";
import { Modal } from "./modal"
import { Button, GuardarButton } from "./botones";
export type Signature = Stroke[];
export interface SignatureState {
    editando: boolean;
    firmaTemp: Signature;
}

export class SignaturePad extends React.Component<FormEditProps<Signature, Signature>, SignatureState> {
    constructor(props) {
        super(props);

        this.state = { editando: false, firmaTemp: [] };
    }

    private handleEditar = () => {
        this.setState({ editando: true, firmaTemp: [] });
    };

    private handleGuardar = () => {
        this.setState({ editando: false });
        if (this.props.onChange) {
            this.props.onChange(this.state.firmaTemp);
        }
    };

    private handleOnChange = (value: Signature) => {
        this.setState({ firmaTemp: value });
    }

    private handleLimpiar = () => {
        this.setState({ firmaTemp: [] });
    }

    private handleCancelar = () => {
        this.setState({ editando: false });
    }

    render() {
        return (
            <div style={{ display: "flex", flexFlow: "column" }}>
                < FormField error={this.props.error} label={this.props.label} >
                    {
                        (this.props.value || []).length > 0 &&
                        <div style={{ width: "100%", height: "200px" }} className="well">
                            <SignatureCanvas zoomToFit={true} value={this.props.value || []} />
                        </div>
                    }
                </FormField>
                {
                    !this.props.isReadOnly &&
                    <div style={{ alignSelf: "flex-start" }}>
                        <Button style="info" iconClass="fa fa-pencil" onClick={this.handleEditar}>Editar firma</Button>
                    </div>
                }
                {
                    this.state.editando &&
                    < Modal
                        title="Escriba su firma aquí:"
                        buttonsWithKey={[
                            <Button key="0" onClick={this.handleLimpiar} iconClass="fa fa-trash" style="danger" >Limpiar</Button>,
                            <GuardarButton key="1" onClick={this.handleGuardar} />
                        ]}
                        fullScreen={true}
                        onClose={this.handleCancelar}
                    >

                        <div style={{ flex: 1, display: "flex" }} >
                            <SignatureCanvas value={this.state.firmaTemp} onChange={this.handleOnChange} />
                        </div>
                    </Modal>
                }
            </div >
        );
    }
}
