import React = require("react");
import { Checkbox, Number, DateTime, Text, Hours } from "./fields/fields";

export interface ComponentProps {
    label?: string;
    isReadOnly?: boolean;
    error?: boolean;
    required?: boolean;
}


export function simpleFields<TValue>(value: TValue, handleChange: <TKey extends keyof TValue>(field: TKey) => (value: TValue[TKey]) => void) {
    type CProps = { label?, value?, onChange?, isReadOnly?, error?};
    const comp = (Component: React.ComponentClass<CProps>, field: keyof TValue, arg?: ComponentProps, extras?: any) => {
        arg = arg || {};
        const label = arg.label;
        const fieldValue: any = value[field];
        const error = arg.error || (!!arg.required && (fieldValue == null || fieldValue == ""));

        return <Component
            label={label == undefined ? field : label}
            onChange={handleChange(field)}
            value={fieldValue}
            isReadOnly={arg.isReadOnly}
            error={error}
            {... (extras || {}) } />
    }

    type TField = keyof TValue;

    const simpleFieldFunc = (component: React.ComponentClass<CProps>) => (field: TField, props?: ComponentProps) => comp(component, field, props);

    return {
        text: (field: TField, props?: ComponentProps & { tip?: string }) => comp(Text, field, props, { placeholder: props && props.tip }),
        datetime: (field: TField, props?: ComponentProps) => comp(DateTime, field, props, { time: true }),
        timespan: (field: TField, props?: ComponentProps) => comp(Hours, field, props, { max: 1000 }),
        daytime: (field: TField, props?: ComponentProps) => comp(Hours, field, props, { max: 24 }),
        date: simpleFieldFunc(DateTime),
        number: simpleFieldFunc(Number),
        check: simpleFieldFunc(Checkbox),
    }
}