import React = require("react");
import card = require("./card");

export interface PaginatedTableState {
    selectedPage: number;
    selectedPageSize: number;
}

/**Propiedades de una tabla */
export interface TableProps {
    /**Cabecera de la tabla. Generalemente esto es un elemento tr */
    header: React.ReactNode;
    /**Elementos de la tabla */
    items?: JSX.Element[];
    /**La tabla no acepta elementos children */
    children?: never;
}

export interface AsyncPaginatedTableProps {
    /**Cabecera de la tabla. Genralmente es un elemento tr */
    header: PromiseLike<JSX.Element> | JSX.Element;
    items?: PromiseLike<(JSX.Element | (() => JSX.Element))[]> | (JSX.Element | (() => JSX.Element))[];
    onChange?: (value: PaginatedTableState) => void;
    value?: PaginatedTableState;
    /**La tabla no acepta elementos children */
    children?: never;
}

/**Propiedades de una tabla asíncrona */
export interface AsyncTableProps {
    /**Cabecera de la tabla. Genralmente es un elemento tr */
    header: PromiseLike<JSX.Element> | JSX.Element;
    items?: PromiseLike<JSX.Element[]> | JSX.Element[];
    /**La tabla no acepta elementos children */
    children?: never;
}

/**Cabecera de una tabla estilizada */
export class TableHeader extends React.Component<{}, {}>{
    render() {
        return (
            <tr className="w3-indigo">
                {this.props.children}
            </tr>
        )
    }
}


/**Componente para un listado general de elementos. Ponga los elementos <tr /> de la tabla en los hijos del elemento*/
export class Table extends React.PureComponent<TableProps, {}>
{
    constructor(props) {
        super(props);
    }

    get items() {
        return this.props.items || (this.props.children as (JSX.Element[] | undefined));
    }

    get TablaVacia() {
        const items = this.items;
        return !items || items.length == 0;
    }

    componentWillReceiveProps(props: TableProps & { children }) {
        if (props.children) {
            console.warn("No mande los hijos de la tabla como hijos del elemento, utilice props.items");
        }
    }

    render() {
        const items = this.items;
        return (
            this.TablaVacia ?
                <card.Card Header="Lista vacía">
                    No hay elementos que mostrar.
                </card.Card> :
                <div className="table-responsive">
                    <table className="table table-striped table-hover table-condensed">
                        <thead>
                            {this.props.header}
                        </thead>
                        <tbody>
                            {items}
                        </tbody>
                    </table>
                </div>
        );
    }
}