import * as React from "react";
import { Tree, TreePath } from "./types";
import { TreeState, TreePathArray } from "./treeState";
import { createSelector } from "reselect";
import { getTreeRows, handleOnCollapsedChange } from "./logic";
import { RowComponentProps, TreeRowComponent } from "./row";
export interface GenericProps<T> {
    value: Tree<T>[];
    state: TreeState;
    onStateChange: (value: TreeState) => void;
    rowRenderer: (value: T, path: TreePath) => React.ReactNode;
}

export type Props = GenericProps<any>

export class TreeComponent extends React.PureComponent<Props> {
    private collapsedPathSelector = (props: Props) => props.state.collapsed;
    private treeSelector = (props: Props) => props.value;
    private treeRowsSelector = createSelector(this.treeSelector, this.collapsedPathSelector,
        (tree, collapsedPath) => getTreeRows([], tree, collapsedPath));

    private stateSelector = (props: Props) => props.state;
    private onStateChangeSelector = (props: Props) => props.onStateChange;

    private handleOnCollapsedChageSelector = createSelector(this.onStateChangeSelector, this.stateSelector,
        (onChange, value) => (collapsed: TreePathArray) => onChange({ ...value, collapsed }));

    private handleOnItemCollapsedChangeSelector = createSelector(this.handleOnCollapsedChageSelector, this.collapsedPathSelector,
        (onChange, collapsed) => (path: TreePath, value: boolean) => onChange(handleOnCollapsedChange(path, value, collapsed)));

    private treeRowPropsSelector = createSelector(this.treeRowsSelector, this.handleOnItemCollapsedChangeSelector,
        (rows, onCollapsed): RowComponentProps[] => rows.map(x => ({
            key: x.path.join("/"),
            path: x.path,
            level: x.path.length,
            collapsed: x.collapsed,
            onCollapsedChange: (value: boolean) => onCollapsed(x.path, value),
            value: x.value
        })));


    private rowRendererSelector = (props: Props) => props.rowRenderer;

    private treeRowComponentsSelector = createSelector(this.treeRowPropsSelector, this.rowRendererSelector,
        (rows, renderer) => rows.map(row =>
            <TreeRowComponent {...row} >
                {renderer(row.value, row.path)}
            </TreeRowComponent>)
    );

    render() {
        const rows = this.treeRowComponentsSelector(this.props);
        return (
            <div>
                {rows}
            </div>
        );
    }
}